//DCF-Uhr mit LCD16x4
//ATmega8 @ 8 MHz
//Autor des Ursprungsprogramms: Johannes Mashighski //Version 2010-02-28-00-11 //AVR-Studio 4.18 Build 692, WinAVR 20100110

//AVR-GCC

#ifndef F_CPU
#error "F_CPU not defined"
#endif

 //////////////////
 //Header Dateien//
 //////////////////

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>
#include <stdlib.h>
#include <util/delay.h>

#include "dcf-clock.h"
#include "lcd-routines.h"
#include "lcd_highlevelfunctions.h"
#include "dcf.h"
#include "1wire.h"
#include "tempmeas.h"



 // volatile bool ausgabe_aktuell=0;

 volatile uint8_t zonenmarke1=0;
 volatile uint8_t zonenmarke2=0;
 volatile uint8_t time_set_at_least_once=0;
 volatile uint8_t time_updated_today=0;
 volatile uint8_t start_dcf_sync=0;
 char s[5];
 //stop / end times
 //start a
 uint8_t start_hour_a	=  8;
 uint8_t start_minute_a	=  0;
 uint8_t start_second_a	=  0;
 //stop a
 uint8_t stop_hour_a	= 20;
 uint8_t stop_minute_a	=  0;
 uint8_t stop_second_a	=  0;
 //start b
 uint8_t start_hour_b	= 10;
 uint8_t start_minute_b	=  0;
 uint8_t start_second_b	=  0;
 //stop b
 uint8_t stop_hour_b	= 22;
 uint8_t stop_minute_b	=  0;
 uint8_t stop_second_b	=  0;


///////////
//Headers//
///////////

 //Main

 int main(void);


 //Sonstiges//

/////////////
//Functions//
/////////////

//Eine fhrende 0 ergnzen bei Zahlen kleiner/gleich 9
void leading_0(uint8_t digit)
{
	if(digit<=9)
	{
		lcd_data('0');
	}
}

/////////////////////////////
//Interrupt Service Routine//
/////////////////////////////
//Der Compare Interrupt Handler wird aufgerufen, wenn TCNT0 = OCR0 = 250-1 ist (250 Schritte), d.h. genau alle 1 ms
ISR (TIMER1_COMPA_vect)
{
	millisecond++;	//Counter fr die Millisekunden
	dcf_read_level();//Der aktuelle Pegel vom DCF-Modul wird eingelesen und in dataDCFPORT den anderen Funktionen zur Verfgung gestellt
// 	counter_lowplusplus();	//Counter fr die Bit-Auswertung (Dauer des low-Pegels), Rcksetzen mu in den Funktionen geschehen
// 	counter_highplusplus();	//Counter fr die Bit-Auswertung (Dauer des high-Pegels), Rcksetzen mu in den Funktionen geschehen
	counter_high++;counter_low++;
	if(millisecond==1000)	//"selbstlaufende" Uhr
	{
// 		lcd_setcursor(5,1);lcd_data('1');
// 		ausgabe_aktuell=0;
		second++;
		millisecond=0;
		if(second==60)
		{
			minute++;
			second=0;
		}
		if(minute==60)
		{
			hour++;
			minute=0;
			if(  ( (hour==2) && (zonenmarke1==0) && (zonenmarke2==1) ) || ( (hour==3) && (zonenmarke1==1) && (zonenmarke2==0) )  )// 		if(synctime())
			{
				dcf_time_reset();time_updated_today=0;start_dcf_sync=1;
			}
		}
		if(hour==24)
		{
			day++;
			week_day++;
			hour=0;
		}
		if(week_day==8)
		{
			week_day=1;
		}
		switch(day)
		{
			case 29:	//Day 28
			{
				leap_year();
				if(month==2 && leapyear==0)
				{
					month++;
					day=1;
				}
			}
			break;

			case 30:	//Day 29
			{
				leap_year();
				if(month==2 && leapyear==1)
				{
					month++;
					day=1;
				}
			}
			break;

			case 31:	//Day 30
			{
				if(month==4 || month==6 || month==9 || month==11)
				{
					month++;
					day=1;
				}
			}
			break;

			case 32:	//Day 31
			{
				if(month==1 || month==3 || month==5 || month==7|| month==8 || month==10 || month==12)
				{
					month++;
					day=1;
				}
			}
			break;

			default: ;break;
		}
		if(month==13)
		{
			year++;
			month=1;
		}
	}

}

////////
//Main//
////////
int main(void)
{
	//Set outputs
	DCFSIGNAL_DDR &= ~(1 << DCFSIGNAL_PIN);
	DCFPOWER_DDR |= (1 << DCFPOWER_PIN);
	OUT_PORT_DDR |= ((1 << OUT_PIN0) | (1 << OUT_PIN1));

	//Set outputs to 0
	OUT_PORT &= ~((1<<OUT_PIN0) | (1<<OUT_PIN1));

	//Initialise LCD
	lcd_init();
	_delay_ms(300);

	// Configure Timer 0
	TCCR1A = 0x00;
	TCCR1B = 0x0B;
	OCR1A = 125-1; //((8000000/64)/1000) = 125
	TIMSK |= (1<<OCIE1A);	//Compare Interrupt erlauben (1)

	//Select sleep_mode for powersaving
	set_sleep_mode(SLEEP_MODE_IDLE);

	//Acticvate Global Interrupts
	sei();
	dcf_time_reset();
	dcf_minute=0;
	dcf_hour=0;
	dcf_timezone=0;
	start_dcf_sync=1;
	time_updated_today=0;
	match_time();
	second=00;
// while(1){
// 	start_meas();
// 	_delay_ms(1100);
// 	read_meas();
// // 	_delay_ms(1100);
// }
	while(1)
	{
		//Set outputs and show time once per second
		if(millisecond==10)
		{
			//Time known (at least vaguely), proceed as you normally would
			if(time_set_at_least_once==1)
			{
				if( ((hour>start_hour_a)||((hour==start_hour_a)&&((minute>start_minute_a)||((minute==start_minute_a)&&(second>=start_second_a))))) && ((hour<stop_hour_a)||((hour==stop_hour_a)&&((minute<stop_minute_a)||((minute==stop_minute_a)&&(second<stop_second_a))))) )
				{
					lcd_updatestatus(0,1);
					OUT_PORT |= (1<<OUT_PIN0);
				}
				else
				{
					lcd_updatestatus(0,0);
					OUT_PORT &= ~(1<<OUT_PIN0);
				}
				if( ((hour>start_hour_b)||((hour==start_hour_b)&&((minute>start_minute_b)||((minute==start_minute_b)&&(second>=start_second_b))))) && ((hour<stop_hour_b)||((hour==stop_hour_b)&&((minute<stop_minute_b)||((minute==stop_minute_b)&&(second<stop_second_b))))) )
				{
					lcd_updatestatus(1,1);
					OUT_PORT |= (1<<OUT_PIN1);
				}
				else
				{
					lcd_updatestatus(1,0);
					OUT_PORT &= ~(1<<OUT_PIN1);
				}
				if(time_updated_today==1)
				{
					show_matched_time(1,2);
					show_matched_date(1,3);
				}
			}
			//System was rebooted: time unknown, don't change outputs
			else
			{
			}
		}
		else if ((time_updated_today==1)&&(millisecond == 200))
		{
			if ((second%10) == 1)
			{
// 				lcd_setcursor(0,4);lcd_string("start");
				start_meas();
			}
			else if ((second%10) == 3)
			{
				lcd_setcursor(0,4);lcd_string("                ");
				read_meas();
			}
		}
		//Should the DCF-Module be synced?
		//While syntime_set_at_least_once==0cingto DCF-signal and detecting the time: always update the LCD
		if(time_updated_today==0)
		{
			if(start_dcf_sync==1)
			{
				if(time_set_at_least_once==0)
				{
					lcd_setfirstsynctext();
				}
				else
				{
					lcd_setresynctext();
				}
				show_matched_time(8,4);
				start_dcf_sync=0;dcf_sync();
			}
 			else
			{
			}
			level_analysis();
			lcd_setcursor(14,2);
			leading_0(bit_number);
			snprintf(s,4,"%d",bit_number);
// 			itoa(bit_number,itoa_buffer,10);
// 			lcd_string(itoa_buffer);
			lcd_string(s);
			level_analysis();
			show_matched_time(8,4);
			level_analysis();
		}
		//If there is nothing else to do: sleep and save power
		else
		{
			OUT_PORT |= (1<<OUT_PIN2); //Laufzeittest
			sleep_mode();
			OUT_PORT &= ~(1<<OUT_PIN2); //Laufzeittest
		}
	}
}
