#include "dcf-clock.h"
#include "1wire.h"
#include "lcd-routines.h"
#include "lcd_highlevelfunctions.h"

void start_meas( void )
{
	if( W1_IN & 1<< W1_PIN ){
		w1_command( CONVERT_T, NULL );
		W1_OUT |= 1<< W1_PIN;
		W1_DDR |= 1<< W1_PIN;			// parasite power on

	}
	else
	{
		lcd_setcursor(0,2);
		lcd_string( "Short Circuit" );
	}
}

#define	DS18S20	0x10
#define THERMOanzahlMax	2	// auch thermoNamen anpassen!

//                      "123456123456"
#define THERMO1WNAMEN	"    TA    TB"	// berschrift, rechtsbndig
//                      " 23.0C 40.3C"
#define THERMO1NameLen	6
#define THERMO1NameOffset	(THERMO1NameLen-2) // hier steht das 1. Zeichen
static	char	thermoNamen[] =	THERMO1WNAMEN;	// ToDo PROGMEM
int16_t	thermoValues[THERMOanzahlMax];
static	uint8_t initReady;

uint8_t valid = 0; 	// Indizes mit gltigen Namen merken, je 1 bit




static uint8_t NameValid( char * name )
{
	// prft, ob Name aus scratch mit Name aus Tabelle bereinstimmt
	// wenn ja, ret = index+1!!!; andernfalls 0
	uint8_t ret = 0;
	char * pC = thermoNamen+THERMO1NameOffset;
	while (ret++ < THERMOanzahlMax)
	{
		if ( *(int16_t*) name== *(int16_t*) pC)	// 2 Byte vergleichen
			return ret;
		pC += THERMO1NameLen;
	}
	return 0;
}


void read_meas( void )
{
lcd_setcursor(0,1);
	unsigned char id[8], diff;
	unsigned char i;
	int16_t tmp;
// 	char s[5];
	// 	uint8_t temp[4] = {0,0,0,0}; // mit CRC
	uint8_t temp[3] = {0,0,0};  // ohne CRC
	int16_t temp_read = 0;

	for( diff = SEARCH_FIRST; diff != LAST_DEVICE; )
	{
		diff = w1_rom_search( diff, id );

		if( diff == PRESENCE_ERR )
		{
			lcd_setcursor(0,2);
			lcd_string( "No Sensor Found" );
			break;
		}
		if( diff == DATA_ERR )
		{
			lcd_setcursor(0,2);
			lcd_string( "Bus Error" );
			break;
		}
		if( id[0] == 0x28 || id[0] == 0x10 )
		{	// temperature sensor
			// lcd_setcursor(0,1);
			// lcd_string( "ID:" );
			//Wert der ID ausgeben
			// for( i = 0; i < 8; i++ )
			// {
			// sprintf( (char*)s, "%02X ", id[i] );
			// lcd_string( (char*)s );
			// }
			char name[2];

			w1_byte_wr( READ );		// read command
			temp[0] = w1_byte_rd();		// temp[0] = byte 0 Temperature LSB
			temp[1] = w1_byte_rd();		// temp[1] = byte 1 Temperature MSB
			name[0] = w1_byte_rd();		// name[0] = byte 2 User Byte 1 (Name)
			name[1] = w1_byte_rd();		// name[1] = byte 3 User Byte 2 (Name)
			w1_byte_rd();			//           byte 4 Reserved (FFh)
			w1_byte_rd();			//           byte 5 Reserved (FFh)
			temp[2] = w1_byte_rd();		// temp[2] = byte 6 Count Remain
							//           byte 7 Count Per C (10h = 16)
			// temp[3] = w1_byte_rd();	// temp[3] = byte 8 CRC

// 			temp_read = temp[0]>>1;
// 			snprintf(s,5,"%0X  ",temp_read);
// 			lcd_string(s);
			if(temp[1] == 0xFF)
			{
				temp_read = (temp[0]>>1)-128;
			}
			else
			{
				temp_read = temp[0]>>1;
			}
			uint8_t nIndex;
			if (!initReady)
			{	// nur beim 1. Durchgang
				// PrintId( id, name );		// erkannte Sensoren mit Namen ausgeben, z.B. auf Uart
				// lcd_setcursor(0,1);		// berschrift in 1. Zeile LCD
				// lcd_string( THERMO1WNAMEN );
			}
			if ((nIndex = NameValid(name)))
			{
				int16_t temp_scratchpad = (10*((16*temp_read)+12-temp[2])+8)/16; //+8: Trick frs Runden
				thermoValues[--nIndex] = (temp_scratchpad);
				valid |= (1<<nIndex);
			}
			else if (initReady==1)			// alle im 0. Durchgang als gltig erkannten sind ausnahmsweise noch gesetzt
			{
				i = 0;
				while ((1<<i++) & valid); 	// 1. freien Platz finden (i ist dann 1 zu gro)
				if (--i<THERMOanzahlMax)	// also vorher 1 abziehen
				{
					valid |= (1<<i);	// jetzt belegt
					i *= THERMO1NameLen;
					i += THERMO1NameOffset;
					w1_command( WRITE, id );
					w1_byte_wr( name[0]=thermoNamen[i++] );
					w1_byte_wr( name[1]=thermoNamen[i] );
					w1_byte_wr( DS18S20 );	// ggf. anpassen fr anderen Sensor!
					w1_command( EE_WRITE, id );
					// PrintId( id, name );	// Namensnderung protokollieren
				}
			}
		}
	}
	//uputsnl( "" );
	lcd_setcursor(0,4);	// Die entsprecheden Stellen in main.c mssen dann natrlich entfernt werden!
	for( i=0; i<THERMOanzahlMax; i++)
	{
		// Error-Detection
		/* es treten gelegentlich Lesefehler auf (Detektor wird nicht gefunden).
		 * Ausgelassene Werte machen viel Arbeit in Excel und verunstalten die Grafik.
		 * Deshalb jetzt: alten Messwert +/- Konstante, z.B. 1=0x10 (1*16).
		 * und das letzte Bit gibt an, ob addiert oder subtrahiert werden muss (abwechselnd)
		 *  Wenn man das nicht braucht, kann man auch einfach einen sonst nicht vorkommenden
		 *  Wert einsetzen, z.B. 0.
		 */
		#define DEFoffset		0x10			// = 1
		#define DEFmask			((int16_t) 0x1)
		tmp=thermoValues[i];

		if (!(valid &(1<<i)))
		{	// keinen neuen Wert gelesen ?
		// dann den alten nehmen und eine wechselnde Markierung anbringen
		// 			if ( temp & DEFmask)	// bit gesetzt?
		// 			{			// dann subtrahieren und bit lschen
		tmp = 850 /*& ~DEFmask*/;
		// 	}
		// 	else 	// addieren und bit setzen
		// 	{
			// 		temp = (temp+DEFoffset) | DEFmask;
		// 	}
		thermoValues[i]=tmp;
		}

		/*tmp sollte sowas sein wie "241" fr 24.1 C*/
// 		itoa(tmp,itoa_buffer,10);
		snprintf(s,5,"% 04d",tmp);
		char *ptr = &s[0];

// 		if(tmp>=0)
// 		{
// 			lcd_data('+');
// // 			if(tmp<100)
// // 			{
// // 				lcd_data(' ');ptr++;
// // 			}
// 		}
// 		else if(tmp>(-100))
// 		{
// 			lcd_data(' ');//ptr++;
// 		}
		lcd_data(*ptr); ptr++;
		lcd_data(*ptr); ptr++;
		lcd_data(*ptr); ptr++;
		lcd_data('.');
		lcd_data(*ptr);
		lcd_data('C');
		lcd_data(' ');
	}
	if (initReady)
	{
		valid = 0;
	}
	// d.h. beim ersten Durchgang stehen lassen fr Erkennung der freien Pltze fr unbekannte Sensoren
	if (initReady<255)
	{// berlauf verhindern
	++initReady;
	}
}